%% Evaluate our diffusion-F measure as an approximation to commute time
% For a real-world graph, we see how close the elements of the Laplacian
% pseudo-inverse are to inverse commute time.

addpath('../../matlab');
%% Real world graph
A = readSMAT('../../data/stanford-cs-sym.smat');
n = size(A,1);
deg = sum(A,1)';
graphname = 'cs-stan';

%% Compute the pseudo-inverse
D = diag(sum(A,1)');
L = D-A;
Lp = pinv(full(L)); % just compute the pseudo-inverse
P = diag(1./deg)*A;


%% Compute commute time values
% build commute time column-by-column... todo: find a matrix way
C = zeros(n);
for i=1:n
    C(:,i) = diag(Lp) + Lp(i,i) - 2*Lp(:,i);
end

%%
% Compute our F-measure
% negative to give relationship with commute time
F = D*Lp;
F = -(F + F');



%% Setup figures
set(0, ...
    'defaultaxesfontsize',   16, ...
    'defaultaxeslinewidth',   1.5, ...
    'defaultlinelinewidth',   2, ...
    'defaultpatchlinewidth',  1.1);

%% Pick a set of random nodes and plot the precision
% The idea is that you run ONE of the various approximations above
% and show it's performance with this section.  
nnodes = 300;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:50];
precisions = zeros(nnodes, length(ks));
for ni=1:nnodes
    curn = nodes(ni);
    [ignore pL] = sort(F(:,curn));
    [ignore pC] = sort(C(:,curn));
    for ki=1:length(ks)
        k = ks(ki);
        precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
    end
end
% Plot mean average precisions
%plot(ks,mean(precisions))
boxplot(precisions,'labels',ks)
ylim([-0.1,1.1])
xlabel('k'); ylabel(sprintf('Precision at k'));
print(gcf,sprintf('%s-commute-vs-diffusion-precision-%i-trials.eps', graphname, nnodes),'-depsc2');

%% Pick a set of random nodes and plot the precision (without the neighbors)
% The idea is that you run ONE of the various approximations above
% and show it's performance with this section.  
nnodes = 300;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:50];
precisions = zeros(nnodes, length(ks));
for ni=1:nnodes
    curn = nodes(ni);
    t = T(:,curn);
    c = C(:,curn);
    t(A(curn,:)>0) = max(t);
    c(A(curn,:)>0) = max(c);
    [ignore pL] = sort(t);
    [ignore pC] = sort(c);
    for ki=1:length(ks)
        k = ks(ki);
        precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
    end
end
% Plot mean average precisions
%plot(ks,mean(precisions))
boxplot(precisions,'labels',ks)
ylim([-0.1,1.1])
xlabel('k'); ylabel(sprintf('Precision at k'));
print(gcf,sprintf('%s-commute-vs-diffusion-precision-noneigh-%i-trials.eps', graphname, nnodes),'-depsc2');