%% Examine localization of Katz scores
% For alpha = 2(1/max(sigma)+10), we check localization of the Katz scores for
% a vertex over a wide range of graphs.  
% Localization is defined as the number of vertices with accounting for 
% 90, 99%, 99.5%, 99.9% of the mass of the Katz score vector.  
% To be completely precise, let
% K = (I-aA) be the Katz matrix
% a Katz vector is K\ei where ei is a unit vector with a 1 in the ith entry
% let x = K\ei, sort the non-zeros of x by decreasing magnitude and 
% check how many account for 90%/99/99.5/99.9% of the mass of the vector, 
% without the diagonal entry of course.

%% Basic setup
% First add information to the paths
addpath('../../matlab'); % add the directory with the katz_push algorithm
addpath('~/dev/gaimc');

%%
% Set the graphs
data_dir = '../../data';
graphs = {...
    'stanford-cs-sym.smat'
    'wiki-Vote-wcc.smat'
    'hollywood-2009-cc.smat'
    'flickr2.smat'
    'arxiv_lcc.smat'
    'ca-AstroPh-cc.smat'
    'ca-CondMat-cc.smat'
    'ca-GrQc-cc.smat'
    'ca-HepPh-cc.smat'
    'ca-HepTh-cc.smat'
    'dblp_lcc.smat'
    'email-Enron-cc.smat'
    'email-EuAll-wcc.smat'
    'soc-Epinions1-wcc.smat'
    'soc-Slashdot0811-wcc.smat'
    'tapir.smat'
    'Stanford3-cc.smat'
    };

%%
% Set the number of trials
ntrials = 100;
ndegtrials = 100;

%% 
% Set a random seed
seed = 0;

%%
% Set a tolerance
tol = 1e-12;
maxiter = 1000;

%% 
% Set the points to evaluate localization
locpts = [0.5 0.75 0.9 0.95 0.99 0.995 0.999 0.9995 0.9999];

%% Run the experimental sweep

results = [];
results_eigs = [];
for gi = 1:length(graphs)
    graph = graphs{gi};
    graphfile = fullfile(data_dir, [graph]);
    A = readSMAT(graphfile);
    A = prepare_graph(A);
    n = size(A,1);
    degs = sum(A,2);
    
    eigs_st = tic;
    [V,D] = eigs(A);
    results_eigs(end+1).dt = toc(eigs_st);
    results_eigs(end).D = diag(D);
    results_eigs(end).V = V;
    alpha = 1/(max(diag(D))+1);
    
    B = speye(n) - alpha*A;
    
    rand('state',seed);
    verts = randperm(n);   
    
    
    
    for ti=1:min(ntrials,n)
        % pick a vertex 
        vertex = verts(ti);
        result = [];
        result.graph = graph;
        result.nverts = n;
        result.nedges = nnz(A);
        result.gi = gi;
        result.alpha = alpha;
        result.trial = ti;
        result.vertex = vertex;
        result.degree = degs(vertex);
        result.tol = tol;
        result.type = 'random';
        st=tic;
        %[x,r,hist,nverts]=katz_push_mex(A,alpha,vertex,tol,maxsteps);
        ei = zeros(n,1);
        ei(vertex) = 1;
        [x,flag,relres,iter] = pcg(B,ei,tol,maxiter);
        x(vertex) = 0;
        dt=toc(st);
        
        ipar = sum(x.^4)/sum(x.^2)^2;
        
        % save data
        result.time = dt;
        result.flag = flag;
        result.relres = relres;
        result.iter = iter;
        
        % evaluate localization
        xs = sort(x,'descend');
        xs = cumsum(xs);
        loc = zeros(length(locpts),1);
        for li=1:length(locpts)
            loc(li) = find(xs>locpts(li)*xs(end),1,'first');
        end
        result.loc = loc;
        result.locpts=locpts;
        result.ipar = ipar;

        fprintf('%10s  %6f  %5i  %6.1f (s)  %5i  %8.2e  %6i  %6i  %6i  %6i\n',...
            graph, alpha, vertex, dt, n, relres, loc(3), loc(6), loc(7), round(1./ipar));

        if isempty(results), results = result; 
        else  results(end+1) = result;
        end
    end
    
    [ignore degp] = sort(degs,'descend');
    for ti=1:min(ndegtrials,n)
        % pick a vertex 
        vertex = degp(ti);
        result = [];
        result.graph = graph;
        result.nverts = n;
        result.nedges = nnz(A);
        result.gi = gi;
        
        result.alpha = alpha;
        result.trial = ti;
        
        result.vertex = vertex;
        result.degree = degs(vertex);
        result.tol = tol;
        result.type = 'degree';
        
        st=tic;
        ei = zeros(n,1);
        ei(vertex) = 1;
        [x,flag,relres,iter] = pcg(B,ei,tol,maxiter);
        x(vertex) = 0;
        dt=toc(st);
        
        ipar = sum(x.^4)/sum(x.^2)^2;
        
        % save data
        result.time = dt;
        result.flag = flag;
        result.relres = relres;
        result.iter = iter;
        
        % evaluate localization
        xs = sort(x,'descend');
        xs = cumsum(xs);
        loc = zeros(length(locpts),1);
        for li=1:length(locpts)
            loc(li) = find(xs>locpts(li)*xs(end),1,'first');
        end
        result.loc = loc;
        result.locpts=locpts;
        result.ipar = ipar;

        fprintf('%10s  %6f  %5i  %6.1f (s)  %5i  %8.2e  %6i  %6i  %6i  %6i\n',...
            graph, alpha, vertex, dt, n, relres, loc(3), loc(6), loc(7), round(1./ipar));

        if isempty(results), results = result; 
        else  results(end+1) = result;
        end
    end
    
    save 'katz_localization_study.mat' results graphs seed tol maxiter locpts;
end

