%% Examine runtime for top-k Katz scores
% For alpha = (1/max(sigma)+1), we check the runtime for 
% using our top-k Katz algorithm

%% Basic setup
% First add information to the paths
addpath('../../matlab'); % add the directory with the katz_push algorithm
addpath('~/dev/gaimc');

%%
% Set the graphs
data_dir = '../../data';
graphs = {...
    'stanford-cs-sym.smat'
    'wiki-Vote-wcc.smat'
    'hollywood-2009-cc.smat'
    'flickr2.smat'
    'arxiv_lcc.smat'
    'ca-AstroPh-cc.smat'
    'ca-CondMat-cc.smat'
    'ca-GrQc-cc.smat'
    'ca-HepPh-cc.smat'
    'ca-HepTh-cc.smat'
    'dblp_lcc.smat'
    'email-Enron-cc.smat'
    'email-EuAll-wcc.smat'
    'soc-Epinions1-wcc.smat'
    'soc-Slashdot0811-wcc.smat'
    'tapir.smat'
    'Stanford3-cc.smat'
    };

%%
% Set the number of trials
ntrials = 100;
ndegtrials = 100;

%% 
% Set a random seed
seed = 0;

%%
% Set a tolerance
tol = 1e-4;
maxiter = 35;


%% Run the experimental sweep

results = [];
results_eigs = [];
for gi = 1:length(graphs)
    graph = graphs{gi};
    graphfile = fullfile(data_dir, [graph]);
    A = readSMAT(graphfile);
    A = prepare_graph(A);
    n = size(A,1);
    degs = full(sum(A,2));
    
    sigma = max(svds(A));
    alpha = 1/(sigma + 1);
    
    rand('state',seed);
    verts = randperm(n);   
    
    for ti=1:min(ntrials,n)
        % pick a vertex 
        vertex = verts(ti);
        result = [];
        result.graph = graph;
        result.nverts = n;
        result.nedges = nnz(A);
        result.gi = gi;
        result.alpha = alpha;
        result.trial = ti;
        result.vertex = vertex;
        result.degree = degs(vertex);
        result.tol = tol;
        result.type = 'random';
        
        st=tic;
        [kp,r,hist,nverts] = katz_push_bal_mex(A,alpha,vertex,1e-4,maxiter*n);
        hist = hist';
        nedges = hist(end,1);
        dt=toc(st);
        
        ipar = sum(kp.^4)/sum(kp.^2)^2;
        
        result.kverts = nverts;
        result.kedges = nedges;
        result.ksteps = length(hist);
        result.ipar = ipar;
        
        % save data
        result.time = dt;
        
      
       
        fprintf('%10s  %6f  %6i  %6i  %6i  %7i  %6i  %6i  %.1f (s)\n',...
            graph, alpha, vertex, degs(vertex),  nverts, nedges, result.ksteps, round(1./ipar), dt);

        if isempty(results), results = result; 
        else  results(end+1) = result;
        end
    end
    
    [ignore degp] = sort(degs,'descend');
    for ti=1:min(ndegtrials,n)
        % pick a vertex 
        vertex = degp(ti);
        result = [];
        result.graph = graph;
        result.nverts = n;
        result.nedges = nnz(A);
        result.gi = gi;
        
        result.alpha = alpha;
        result.trial = ti;
        
        result.vertex = vertex;
        result.degree = degs(vertex);
        result.tol = tol;
        result.type = 'degree';
        
        st=tic;
        [kp,r,hist,nverts] = katz_push_bal_mex(A,alpha,vertex,1e-4,maxiter*n);
        hist = hist';
        nedges = hist(end,1);
        dt=toc(st);
        
        ipar = sum(kp.^4)/sum(kp.^2)^2;
        
        result.kverts = nverts;
        result.kedges = nedges;
        result.ksteps = length(hist);
        result.ipar = ipar;
        
        % save data
        result.time = dt;
       
        fprintf('%10s  %6f  %6i  %6i  %6i  %7i  %6i  %6i  %.1f (s)\n',...
            graph, alpha, vertex, degs(vertex),  nverts, nedges, result.ksteps, round(1./ipar), dt);

        if isempty(results), results = result; 
        else  results(end+1) = result;
        end
    end
    
    save 'katz_runtime_all.mat' results graphs seed tol maxiter;
end

%%
load katz_runtime_all

nresults = length(results);
ipars = [];
kverts = [];
kedges = [];
dts = [];

vtypes = [];

gs = [];

nverts = [];
nedges = [];
for i=1:length(results)
    if strcmp(results(i).type,'degree'), vtype = 1; 
    else vtype = 2;
    end
    
    ipars = [ipars 1./results(i).ipar];
    gs = [gs results(i).gi];
    vtypes = [vtypes vtype];
    
    dts = [dts results(i).time];
    
    kverts = [kverts results(i).kverts];
    kedges = [kedges results(i).kedges];
    
    nverts(results(i).gi) = results(i).nverts;
    nedges(results(i).gi) = results(i).nedges;
end

vtype = vtype(:);
gs = gs(:);
dts = dts(:);

ddt25 = accumarray(gs(vtype==1),dts(vtype==1),[],@(x) prctile(x,25));
ddt50 = accumarray(gs(vtype==1),dts(vtype==1),[],@(x) prctile(x,50));
ddt75 = accumarray(gs(vtype==1),dts(vtype==1),[],@(x) prctile(x,75));

rdt25 = accumarray(gs(vtype==1),dts(vtype==2),[],@(x) prctile(x,25));
rdt50 = accumarray(gs(vtype==1),dts(vtype==2),[],@(x) prctile(x,50));
rdt75 = accumarray(gs(vtype==1),dts(vtype==2),[],@(x) prctile(x,75));


[ignore p] = sort(nverts);

for pgi=1:length(graphs)
    gi = p(pgi);
    fprintf('%20s & %7i & %5.1f & %3.1f & %6.1f & %6.1f & %6.1f \\\\\n', ...
        shortnames{gi}, nverts(gi), nedges(gi)/nverts(gi), mins(gi),  means(gi), medians(gi), maxs(gi));
end


