/** @file matvec_mex.c
 * A mex-file implementation of a standard matvec
 */

/*
 * David F. Gleich
 * Copyright, Sandia Corporation, 2011
 */

/** 
 * History
 * -------
 * :2011-03-23: Initial coding based on katz_push_bal_mex.c
 */

#include <mex.h>
#include <matrix.h>

#include <string.h>
#include <math.h>

#include <sys/types.h>
#include <sys/timeb.h>
double sf_time()
{
#if defined(_WIN32) || defined(_WIN64)
  struct __timeb64 t; _ftime64(&t);
  return (t.time*1.0 + t.millitm/1000.0);
#else
  struct timeval t; gettimeofday(&t, 0);
  return (t.tv_sec*1.0 + t.tv_usec/1000000.0);
#endif
}

/** Compute A'*X
 * where X = [x1,x2] is stored by columns
 */
void dualmatvec(mwSize N, const mwIndex* ai, const mwIndex* aj,
            double* x, double *y)
{
    mwIndex i;
    for (i=0; i<N; ++i) {
        double yi = 0.;
        double yi2 = 0.;
        mwIndex nzi;
        for (nzi=ai[i]; nzi<ai[i+1]; ++nzi) {
            mwIndex j = aj[nzi];
            yi += x[j];
            yi2 += x[j+N];
        }
        y[i] = yi;
        y[i+N] = yi2;
    }
}
        


/* matvec_mex:
 *   [y,dt] = dualmatvec_mex(A,x,ntrials)
 */
void mexFunction(int nargout, mxArray *pargout[],
                 int nargin, const mxArray* pargin[])
{
    const mxArray *argA, *argx, *argntrials;
    mxArray *argy, *dts;
    mwIndex trials, i;
    mwIndex *ai, *aj;
    mwSize n;
    double *x, *y, *hist;
    
    if (nargin < 3) {
        mexErrMsgIdAndTxt("MATLAB:nargchk:notEnoughInputs",
            "Not enough input arguments.");
    } else if (nargin > 3) {
        mexErrMsgIdAndTxt("MATLAB:nargchk:tooManyInputs",
            "Too many input arguments.");
    }
    
    argA = pargin[0];
    argx = pargin[1];
    argntrials = pargin[2];
    

    /* validate arguments */
    if (!mxIsSparse(argA)) {
        mexErrMsgIdAndTxt("matvec_mex:sparseRequired",
            "The input A argument must be a sparse matrix.");
    }
    if (mxGetNumberOfElements(argx) != 2*mxGetN(argA)) {
        mexErrMsgIdAndTxt("matvec_mex:vectorRequired",
            "The input x argument must be a scalar.");
    }
    if (!mxIsScalar(argntrials)) {
        mexErrMsgIdAndTxt("matvec_mex:scalarRequired",
            "The input ntrials argument must be a scalar.");
    }
    
    /* Load the sparse matrix */
    ai = mxGetJc(argA);
    aj = mxGetIr(argA);
    n = mxGetN(argA);
    
    trials = (mwIndex)mxGetScalar(argntrials);
    mxAssert(trials >= 0, "trials >= 0");
       
    argy = mxCreateDoubleMatrix(n,2,mxREAL);
    dts = mxCreateDoubleMatrix(1,trials+1,mxREAL);
    
    x = mxGetPr(argx);
    y = mxGetPr(argy);
    hist = mxGetPr(dts);
    
    for (i=0; i<trials+1; ++i) {
        double t0,dt;
        t0 = sf_time();
        dualmatvec(n, ai, aj, x, y);
        dt = sf_time() - t0;
        hist[i] = dt;
    }
    
    nargout = 2;
    pargout[0] = argy;
    pargout[1] = dts;
}
